#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Sun Oct 29 11:48:28 2017

@author: rene
"""
 
import multiprocessing
import warnings
import os
from SimulationThread import SimulationThread
import time

class Conductor(multiprocessing.Process):
    runningThreads = 0
    numberOfThreads = 0
    runParameters = None
    resultPath = None
    threads = None
    runs = 0
   
    def __init__(self, runParameters, numberOfThreads, resultPath="results"):
        super(Conductor, self).__init__()
        self.exit = multiprocessing.Event()

        self.runParameters = runParameters
        self.numberOfThreads = numberOfThreads
        
        #remove trailing /
        if resultPath[-1] == "/":
            resultPath = resultPath[:-1]
        self.resultPath =  resultPath
        
        self.threads = [None]*numberOfThreads
        
    def run(self):
        for i in range(self.numberOfThreads):
            self.startNewThread(threadNo=i)
            time.sleep(0.5)
        
        while not(self.exit.is_set()) and len(self.runParameters) > 0:
            for i, g in enumerate(self.threads):
                if not(g is None):
                    if not(g.is_alive()):
                        print("Simulation %.f has finished!"%i)
                        self.threads[i] = None
                        self.startNewThread(i)
                    else:
                        g.join(0.01)
        
        
        print("Signaling all threads to finish...")
        
        if self.exit.is_set():
            for i, g in enumerate(self.threads):
                if not(g is None):
                    g.terminate()
        
        print("Waiting for remaining threads to finish...")
        
        for i, g in enumerate(self.threads):
            if not(g is None):
                g.join()
        
        print("All simulations are done!")

    def startNewThread(self, threadNo):
        if not(self.exit.is_set()):
            if len(self.runParameters) > 0:
                print("----------------------------------------")
                print("Starting new Simulation-Thread! ("+str(threadNo)+")")
                runP = self.runParameters.pop()
                self.runs += 1
                print("Parameters are:\tTDP: "+runP['TDP'])
                print("\t\tconcentration: %.2f"%runP['conc'])
                print("\t\tvolumeFraction=%.3f"%runP['vol'])
                print("\t\tbeta⁻¹=%.f"%(1/runP['beta']))
                
                folder = self.resultPath+"/"+runP['TDP']+"/%.2f_%.3f_%2.f"%(runP['conc'], runP['vol'], 1/runP['beta'])
                if not os.path.exists(folder):
                    os.makedirs(folder)
                else:
                    warnings.warn("Simulation has run before!")
                
                g = SimulationThread(runParams=runP, folder=folder)
                
                #cb = lambda: self.threadDone(threadNo=threadNo)
                #g.addListener(cb)
                g.start()
                
                self.threads[threadNo] = g
                print("----------------------------------------")
            else:
                print("No more simulations to run! Waiting for other threads to finish...")
        else:
            print("Shutdown, not starting any new Threads!")
        
    def is_alive(self):
        return self.is_running()
        
    def is_running(self):
        a=False
        for t in self.threads:
            if not(t is None):
                a=a or t.is_alive()
        
        return a
    
    def terminate(self):
        print("Conductor terminate initiated")
        self.exit.set()
    