#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Sun Oct 29 11:58:11 2017

Copy-Paste this file to create different simulation sets.

@author: rene
"""
from Generator import TDP
from Conductor import Conductor

import sys
print(sys.argv)

#Read in the number of cores to use (== threads to start)
if '--cores' in sys.argv:
    cores=int(sys.argv[sys.argv.index('--cores')+1])
else:
    cores = 4
print("Producing %.f threads."%cores)

#Read in the output folder for the results
if '--output' in sys.argv:
    folder=sys.argv[sys.argv.index('--output')+1]
    
    #Remove trailing \ or /
    if "/" in folder or "\\" in folder:
        if folder[-1] == "/" or folder[-1] == "\\":
            folder = folder[:-1]
else:
    folder = "results"
print("Writing results to "+folder+"/")

#Read in the file location for the file specifying the run parameters
if len(sys.argv)>1:
    file = sys.argv[-1]
else:
    file = 'input.csv'
print("Reading runparameters from "+file+"")


##################################################################
#Read in the run parameters and verify them.
concDefault = 1
volDefaul = 0.5
betaDefault = 1/20
iterationsDefault = 3e5
drawingIntervalDefault = 1e4


defaults = {
        'TDP':TDP.RANDOM_HYDROPHOB, 'conc':1, 'vol':0.5, 'beta':1/20,
        'drawingInterval':1e4, 'iterations': 3e5,
        }

def TDPtype(x):
    if not(x in TDP.sequences):
        print("TDP type "+x+" not known.")
        print("Available TDP types are: ",list(TDP.sequences.keys()))
        return TDP.RANDOM_HYDROPHOB
    else:
        return x
        
types = {
        'TDP':TDPtype, 'conc':float, 'vol':float, 'beta':float,
        'drawingInterval':lambda x:int(float(x)), 'iterations': lambda x:int(float(x)),
        }

import csv
import copy
runParameters = []
with open(file) as csvfile:
    reader = csv.DictReader(csvfile)
    for row in reader:
        params = copy.deepcopy(defaults)
        for k in row.keys():
            if k in defaults:
                v = row[k]
                if k in types:
                    v = types[k](v)
                params[k] = v
        
        runParameters.append(params)
    
    for k in row.keys():
        if not k in defaults:
            print("Unknown key used: "+k)
            print("Available keys are: ",list(defaults.keys()))

print("Found %.f parameter sets to run."%len(runParameters))
##################################################################

print("Starting simulations.")

c = Conductor(runParameters=runParameters, numberOfThreads=cores, resultPath=folder)
c.start()
print("You can terminate the simulations by typing c.terminate()")
